From bce031b4e9257543f5cb58b1c935f0e002b162b8 Mon Sep 17 00:00:00 2001
From: Holger Hans Peter Freyther <zecke@selfish.org>
Date: Tue, 22 Sep 2009 12:39:15 +0200
Subject: [PATCH 4/4] network: Parse RFC1123 directly and don't go through QLocale

The QDateTimeParser forces too many allocations, creates
a QString, converts it to lower case... This makes the
replyHeaderChanged show up in the profile of loading_standalone.

The new RFC1123 parsing is 10 times faster on a macbook with
a core duo.
---
 src/network/access/qnetworkrequest.cpp |   76 +++++++++++++++++++++++++++++---
 1 files changed, 69 insertions(+), 7 deletions(-)

diff --git a/src/network/access/qnetworkrequest.cpp b/src/network/access/qnetworkrequest.cpp
index 523712b..70dcfa6 100644
--- a/src/network/access/qnetworkrequest.cpp
+++ b/src/network/access/qnetworkrequest.cpp
@@ -772,6 +772,65 @@ void QNetworkHeadersPrivate::parseAndSetHeader(const QByteArray &key, const QByt
     }
 }
 
+int name_to_month(const char* month_str)
+{
+    switch (month_str[0]) {
+    case 'J':
+        switch (month_str[1]) {
+        case 'a':
+            return 1;
+            break;
+        case 'u':
+            switch (month_str[2] ) {
+            case 'n':
+                return 6;
+                break;
+            case 'l':
+                return 7;
+                break;
+            }
+        }
+        break;
+    case 'F':
+        return 2;
+        break;
+    case 'M':
+        switch (month_str[2] ) {
+        case 'r':
+            return 3;
+            break;
+        case 'y':
+            return 5;
+            break;
+        }
+        break;
+    case 'A':
+        switch (month_str[1]) {
+        case 'p':
+            return 4;
+            break;
+        case 'u':
+            return 8;
+            break;
+        }
+        break;
+    case 'O':
+        return 10;
+        break;
+    case 'S':
+        return 9;
+        break;
+    case 'N':
+        return 11;
+        break;
+    case 'D':
+        return 12;
+        break;
+    }
+
+    return 0;
+}
+
 QDateTime QNetworkHeadersPrivate::fromHttpDate(const QByteArray &value)
 {
     // HTTP dates have three possible formats:
@@ -787,16 +846,19 @@ QDateTime QNetworkHeadersPrivate::fromHttpDate(const QByteArray &value)
         // no comma -> asctime(3) format
         dt = QDateTime::fromString(QString::fromLatin1(value), Qt::TextDate);
     } else {
-        // eat the weekday, the comma and the space following it
-        QString sansWeekday = QString::fromLatin1(value.constData() + pos + 2);
 
-        QLocale c = QLocale::c();
-        if (pos == 3)
-            // must be RFC 1123 date
-            dt = c.toDateTime(sansWeekday, QLatin1String("dd MMM yyyy hh:mm:ss 'GMT"));
-        else
+        if (pos == 3) {
+            char month_name[4];
+            int day, year, hour, minute, second;
+            if (sscanf(value.constData(), "%*3s, %d %3s %d %d:%d:%d 'GMT'", &day, month_name, &year, &hour, &minute, &second) == 6)
+                dt = QDateTime(QDate(year, name_to_month(month_name), day), QTime(hour, minute, second));
+        } else {
+            QLocale c = QLocale::c();
+            // eat the weekday, the comma and the space following it
+            QString sansWeekday = QString::fromLatin1(value.constData() + pos + 2);
             // must be RFC 850 date
             dt = c.toDateTime(sansWeekday, QLatin1String("dd-MMM-yy hh:mm:ss 'GMT'"));
+        }
     }
 #endif // QT_NO_DATESTRING
 
-- 
1.6.3.3

